/****************************************************************************
 * Copyright (C) 2012 FIX94
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ogcsys.h>
#include <malloc.h>
#include <unistd.h>
#include <sdcard/wiisd_io.h>
#include <ogc/machine/processor.h>
#include "fs.h"
#include "fat.h"

extern "C" 
{
extern void exit(int status);
extern s32 MagicPatches(s32);
extern void __exception_setreload(int t);
}

#define SD_CERT_PATH	"sd:/cert.sys"
#define HW_AHBPROT		((vu32*)0xCD800064)
#define MEM_PROT		((vu32)0xCD8B420A)

/* Thanks to postloader for that patch */
#define ES_MODULE_START	((u16*)0x939F0000)
static const u16 ticket_check[] = {
    0x685B,          // ldr r3,[r3,#4] ; get TMD pointer
    0x22EC, 0x0052,  // movls r2, 0x1D8
    0x189B,          // adds r3, r3, r2; add offset of access rights field in TMD
    0x681B,          // ldr r3, [r3]   ; load access rights (haxxme!)
    0x4698,          // mov r8, r3     ; store it for the DVD video bitcheck later
    0x07DB           // lsls r3, r3, #31; check AHBPROT bit
};
void PatchAHB()
{
	write16(MEM_PROT, 0);
	for(u16 *patchme = ES_MODULE_START; patchme < ES_MODULE_START + 0x4000; patchme++) 
	{
		if(!memcmp(patchme, ticket_check, sizeof(ticket_check))) 
		{
			// write16/uncached poke doesn't work for this. Go figure.
			patchme[4] = 0x23FF; // li r3, 0xFF
			DCFlushRange(patchme + 4, 2);
			break;
		}
	}
	write16(MEM_PROT, 1);
}

void InitVideo()
{
	VIDEO_Init();
	GXRModeObj *rmode = VIDEO_GetPreferredMode(NULL);
	void *xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_ClearFrameBuffer(rmode, xfb, COLOR_BLACK);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if(rmode->viTVMode&VI_NON_INTERLACE)
		VIDEO_WaitVSync();
	CON_InitEx(rmode, 24, 32, rmode->fbWidth - (32), rmode->xfbHeight - (48));
}

void ExitProgram(void)
{
	ISFS_Deinitialize();
	MagicPatches(0);
	sleep(5);
	exit(0);
}

int main()
{
	__exception_setreload(10);
	InitVideo();
	printf("Simple cert.sys Extractor v0.1 by FIX94\n \n");
	if(*HW_AHBPROT != 0xFFFFFFFF)
	{
		printf("AHB Protection is not disabled! Exiting...\n");
		exit(0);
	}
	printf("Patching IOS ES Module\n");
	PatchAHB();
	printf("Reloading to IOS58\n");
	IOS_ReloadIOS(58);

	printf("Patching IOS NAND Rights...\n");
	MagicPatches(1); //It's a kind of magic :P
	printf("Initing ISFS\n");
	ISFS_Initialize();
	printf("Trying to grab the cert.sys...\n");
	char filepath[ISFS_MAXPATH] ATTRIBUTE_ALIGN(32);
	sprintf(filepath, "/sys/cert.sys");
	u32 certSize = 0;
	u8 *certBuffer = ISFS_GetFile((u8*)&filepath, &certSize, -1);
	if(certSize == 0 || certBuffer == NULL)
	{
		printf("Failed to get the cert.sys!\n");
		ExitProgram();
	}

	printf("Got File, Mounting SD Card to save it...\n");
	bool isMounted = fatMountSimple("sd", &__io_wiisd);
	if(!isMounted)
	{
		printf("Failed to Mount SD Card!\n");
		free(certBuffer);
		ExitProgram();
	}
	printf("Checking if the File already exist\n");
	FILE *fp = fopen(SD_CERT_PATH, "rb");
	if(fp != NULL)
	{
		printf("File already exist, aborting...\n");
		fclose(fp);
		fatUnmount("sd");
		free(certBuffer);
		ExitProgram();
	}
	printf("File doesn't exist, Writing File to SD Card...\n");
	fp = fopen(SD_CERT_PATH, "wb");
	u32 writtenSize = fwrite(certBuffer, 1, certSize, fp);
	fclose(fp);
	if(writtenSize != certSize)
	{
		printf("Something went wrong, please check your SD Card for errors!\n");
		remove(SD_CERT_PATH);
	}
	else
		printf("Done! Back to The Homebrew Channel...\n");
	fatUnmount("sd");
	free(certBuffer);
	ExitProgram();
	return 0;
}
